@@ -6,4 +6,4 @@ operator = get_operator()
 
 for i, line in enumerate(sys.stdin):
     old_hash, new_hash = line.split()
-    operator.track_commit(old_hash, new_hash, sys.argv[1])
+    operator.track_rebase(old_hash, new_hash)
