@@ -1,25 +1,30 @@
 from importlib_resources import files
 
-import os
-
-from .parsing import test_skill
+from .parsing import test_ancestry
 from .parsing import level_json
 from .parsing import parse_spec
+from .parsing import name_from_map
+from .parsing import get_non_merges
+from .parsing import name_merges
 
+from .user_messages import print_user_file
 from .user_messages import print_user_message
 from .user_messages import show_level_name
-from .user_messages import print_goal
-from .user_messages import simulate_goal
 from .user_messages import show_tree
 from .user_messages import default_fail
 from .user_messages import level_complete
 from .user_messages import skill_complete
 from .user_messages import all_levels_complete
+from .user_messages import solution_print_header
+from .user_messages import no_solutions_available
+
+from gitgud import operations
 
 
 class Level:
-    def __init__(self, name):
+    def __init__(self, readable_name, name):
         self.name = name
+        self.readable_name = readable_name
         self.skill = None
         self.next_level = None
         self.prev_level = None
@@ -32,17 +37,16 @@ class Level:
 
     def full_name(self):
         return '{} {}'.format(self.skill.name, self.name)
-    
-    def _setup(self, file_operator):
+
+    def _setup(self):
         pass
 
-    def setup(self, file_operator):
-        show_level_name(self)
-        self._setup(file_operator)
+    def setup(self):
+        self._setup()
         self.post_setup()
-    
+
     def post_setup(self):
-        pass
+        show_level_name(self)
 
     def instructions(self):
         pass
@@ -52,12 +56,15 @@ class Level:
 
     def status(self):
         show_level_name(self)
-    
-    def _test(self, file_operator):
-        pass
 
-    def test(self, file_operator):
-        if self._test(file_operator):
+    def has_ever_been_completed(self):
+        return self._test()
+
+    def _test(self):
+        raise NotImplementedError
+
+    def test(self):
+        if self._test():
             self.test_passed()
         else:
             self.test_failed()
@@ -72,24 +79,34 @@ class Level:
 
     def test_failed(self):
         default_fail()
-          
+
 
 class BasicLevel(Level):
-    def __init__(self, name, skill_package):
-        super().__init__(name)
+    def __init__(self, readable_name, name, skill_package):
+        super().__init__(readable_name, name)
 
         self.level_dir = files(skill_package).joinpath('_{}/'.format(name))
 
         self.setup_spec_path = self.level_dir.joinpath('setup.spec')
         self.test_spec_path = self.level_dir.joinpath('test.spec')
+
+        self.instructions_path = self.level_dir.joinpath('instructions.txt')
+        self.goal_path = self.level_dir.joinpath('goal.txt')
+
         self.passed_path = self.level_dir.joinpath('passed.txt')
 
-        self.goal_path = self.level_dir.joinpath('goal.txt')
-        self.instructions_path = self.level_dir.joinpath('instructions.txt')
         if not self.instructions_path.exists():
             self.instructions_path = self.goal_path
 
-    def _setup(self, file_operator):
+        self.solution_path = self.level_dir.joinpath('solution.txt')
+        self.solution_commands = self.solution_list()
+
+    def display_message(self, message_path):
+        path = self.level_dir.joinpath(message_path)
+        print_user_file(path)
+
+    def _setup(self):
+        file_operator = operations.get_operator(initialize_repo=True)
         commits, head = parse_spec(self.setup_spec_path)
         file_operator.create_tree(commits, head)
 
@@ -104,7 +121,7 @@ class BasicLevel(Level):
         file_operator.write_last_commit(latest_commit)
 
     def post_setup(self):
-        simulate_goal(self)
+        self.display_message('goal.txt')
         show_tree()
 
     def instructions(self):
@@ -114,22 +131,57 @@ class BasicLevel(Level):
             else:
                 print(line.strip())
 
-    def goal_str(self):
-        return self.goal_path.read_text().strip()
-
     def goal(self):
-        print_goal(self)
+        self.display_message("goal.txt")
 
-    def _test(self, file_operator):
+    def solution_list(self):
+        solution_commands = []
+
+        for command in self.solution_path.read_text().split('\n'):
+            if command and command.strip()[0] != "#":
+                solution_commands.append(command)
+
+        return solution_commands
+
+    def solution(self):
+        solution = self.solution_list()
+        if not solution:
+            no_solutions_available()
+        else:
+            solution_print_header(self)
+            for command in solution:
+                print(' '*4 + command)
+
+    def _test(self):
+        file_operator = operations.get_operator()
         commits, head = parse_spec(self.test_spec_path)
+
+        # Get commit trees
         test_tree = level_json(commits, head)
         level_tree = file_operator.get_current_tree()
-        return test_skill(level_tree, test_tree)
+
+        # Get commit info
+        non_merges = get_non_merges(level_tree)
+
+        # Name known commits
+        known_commits = file_operator.get_known_commits()
+        name_from_map(level_tree, known_commits)
+
+        # Name rebases and cherrypicks
+        known_non_merges = {commit_hash: name
+                            for commit_hash, name in known_commits.items()
+                            if name[:1] != 'M'}
+        diff_map = file_operator.get_copy_mapping(non_merges, known_non_merges)
+        name_from_map(level_tree, diff_map)
+
+        # Name merges
+        name_merges(level_tree, test_tree)
+
+        # Test for similarity
+        return test_ancestry(level_tree, test_tree)
 
     def test_passed(self):
         if self.passed_path.exists():
             print_user_message(self.passed_path.read_text())
         else:
             super().test_passed()
-            
-
