@@ -10,13 +10,11 @@ def test_load(gg):
         ('git gud load rampup', all_skills["rampup"]["1"]),
         ('git gud load 2 detaching', all_skills["2"]["detaching"]),
         ('git gud load rampup 4', all_skills["rampup"]["4"]),
-        ('git gud load 3-octopus', all_skills["3"]["octopus"]),
+        ('git gud load 4-octopus', all_skills["4"]["octopus"]),
         ('git gud load rampup-4', all_skills["rampup"]["4"]),
         ('git gud load -2', all_skills["rampup"]["2"])
     ]
-    
+
     for command, level in load_tests:
         subprocess.call(command, shell=True)
-        assert level == gg.file_operator.get_level()
-    
-    
\ No newline at end of file
+        assert level == gg.get_level()
